#pragma once

#include <Wt/WApplication.h>
#include <cmath>

using namespace Wt;
using namespace std;


struct sin {
    double a;
    double w;
    double f;

    Chart::WDataSeries* ds;
    WContainerWidget* el;

    double get(double t) const
    { return a*cos(w*t+f); }

    bool operator ==(const sin& other) const {
        return a == other.a && w == other.w && f == other.f;
    }

    bool operator <(const sin& other) const {
        return (a+w+f) < (other.a + other.w + other.f);
    }
};
typedef struct sin Sin;

class SinModel;
class FourierModel;
class RestoreModel;

class MyApplication : public WApplication
{
public:
    MyApplication(const WEnvironment& env);

private:
    Chart::WCartesianChart* chart;
    shared_ptr<SinModel> roughModel;
    shared_ptr<SinModel> detailedModel;
    shared_ptr<SinModel> discreteModel;
    Chart::WDataSeries* pointsDS;

    shared_ptr<RestoreModel> restoreModel;
    vector<pair<double, double>> fourier_data;
    Chart::WDataSeries* restoredDS;
    Chart::WDataSeries* restoredDS_points;

    WCheckBox* lineCheck;
    WCheckBox* pointsCheck;

    Chart::WCartesianChart* fourierChart;
    shared_ptr<FourierModel> fourierModel;


    WContainerWidget* list;
    WCheckBox* showAll;
    size_t checkCount = 0;

    vector<Sin> sinuses;

    void addSin(Sin s);
    void removeSin(int index);
};
